 /*	
	dataspy network x version 0.5 beta
	Copyright (C) 2002 dataspy dot net INC

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */

 /*

	HexToText.cpp - converts from hex to text n vice versa

	Last revised : 31st March 2002

  */

/* 
	Converts 1 byte to 1.5 readable chars (2 bytes to 3 chars)
  
	Chars used are:
					a-z			: 26 (00-25)
					0-9			: 10 (26-35)
					{}|^-		: 5  (36-40)
					[]\`_
					  

  For a total of 41 (0-40) - these chars are chosen because they are valid for use in IRC nicknames

  possible to use 3 chars instead of 4 because:  ln 41^3 / ln 2 > 16 bits = at least 2 bytes...

 */


#include <windows.h>
#include "..\include\hextotext.h"

short GetValue(char c)
{
	if (c >= '0' && c <= '9') return c - '0';

	if (tolower(c) >= 'a' && tolower(c) <= 'z') return tolower(c) - 'a' + 10;

	if (c == VAL_EX_1_A || c == VAL_EX_1_B) return VAL_EX_1;
	if (c == VAL_EX_2_A || c == VAL_EX_2_B) return VAL_EX_2;
	if (c == VAL_EX_3_A || c == VAL_EX_3_B) return VAL_EX_3;
	if (c == VAL_EX_4_A || c == VAL_EX_4_B) return VAL_EX_4;
	if (c == VAL_EX_5_A || c == VAL_EX_5_B) return VAL_EX_5;
	

	return -1;
}

char GetChar(short v)
{
	register a = (GetTickCount() + v) % 2;

	if (v >= 0 && v <= 9) return '0' + v;

	if (v >= 10 && v <= 35) {
		if (a) 
			return 'a' -10 + v;
		else
			return 'A' - 10 + v;
	}

	if (v == VAL_EX_1) if(a) return VAL_EX_1_A; else return VAL_EX_1_B;
	if (v == VAL_EX_2) if(a) return VAL_EX_2_A; else return VAL_EX_2_B;
	if (v == VAL_EX_3) if(a) return VAL_EX_3_A; else return VAL_EX_3_B;
	if (v == VAL_EX_4) if(a) return VAL_EX_4_A; else return VAL_EX_4_B;
	if (v == VAL_EX_5) if(a) return VAL_EX_5_A; else return VAL_EX_5_B;
	
	return -1;
}

int BToC(BYTE *b)
{
	unsigned short val = 0,tmp; 

	memcpy(&val,b,2);

	tmp = val / MAX_SQUARED;
	b[2] = GetChar(tmp);
	val -= MAX_SQUARED * tmp;

	tmp = val / MAX;
	b[1] = GetChar(tmp);
	val -= MAX * tmp;

	b[0] = GetChar(val);

	if (val > MAX) 
		return 1;
	else
		return 0;
}

int CToB(BYTE *b)
{
	unsigned short val =0;

	val += GetValue(b[2]) * MAX_SQUARED;
	val += GetValue(b[1]) * MAX;
	val += GetValue(b[0]);

	memcpy(b,&val,2);
	
	return 0;
}
/*
//Converts an IPv4 ip address and port to a 9 char nickname. Case is ignored.
int IpAndPortToNickname(char* buffer,unsigned long ip,unsigned short port)
{
	char ipn[6];
	int c=6;

	memcpy(ipn + 2,&ip,4);
	memcpy(ipn,&port,2);

	XOR(ipn,c)

	BToC((BYTE*)memcpy(buffer,ipn,2));
	BToC((BYTE*)memcpy(buffer+3,ipn+2,2));
	BToC((BYTE*)memcpy(buffer+6,ipn+4,2));
	
	buffer[9] = 0;
	return 0;
}

int NicknameToIpAndPort(char* buffer,unsigned long *ip,unsigned short *port)
{
	char tmp[6];
	int c = 6;
	CToB((BYTE*)buffer);
	CToB((BYTE*)buffer+3);
	CToB((BYTE*)buffer+6);

	memcpy(tmp,buffer,2);
	memcpy(tmp+2,buffer+3,2);
	memcpy(tmp+4,buffer+6,2);

	XOR(tmp,c)
	
	memcpy(ip,tmp+2,4);
	memcpy(port,tmp,2);	
	
	return 0;
}
*/
BOOL TextToHex(char *buffer,char **data,unsigned short *data_len)
{
	char *p,*b,t[4];
	unsigned short out_len,len;

	if (!buffer || !data || !data_len) {
		return FALSE;
	}

	len = strlen(buffer);
	
	if (!len) 
		return FALSE;

	out_len = len / 3 * 2;
	*data = (char*)malloc(out_len + 1);

	if (!*data)
		return FALSE;

	p = buffer;
	b = *data;

	while (len > 2) {
		memset(t,0,sizeof(t));
		memcpy(t,p,3);
		
		CToB((unsigned char*)t);

		memcpy(b,t,2);

		p+= 3;
		len -=3;
		b+= 2;		
	}

	*b =0;
	*data_len = out_len;
	return TRUE;
}

BOOL HexToText(char *data,unsigned short *data_len,char **buffer)
{
	char *p,*b,t[4];

	if (!data || !data_len || !buffer) {
		return FALSE;
	}

	(*buffer) = (char*)malloc((*data_len / 2 * 3) + 4);

	if (!*buffer)
		return FALSE;
	
	p = data;
	b = *buffer;

	while (*data_len > 1) {
		memset(t,0,sizeof(t));
		memcpy(t,p,2);
		
		BToC((unsigned char*)t);

		memcpy(b,t,3);

		p+= 2;
		*data_len -=2;
		b+= 3;		
	}

	if (*data_len) { /* if (*data_len == 1) { */
		memset(t,0,sizeof(t));
		memcpy(t,p,1);
		
		BToC((unsigned char*)t);

		memcpy(b,t,3);

		p+= 2;
		b+= 3;
	}

	*b =0;
	return TRUE;
}
